@@ -1,6 +1,6 @@
 ;;; el_client.el --- XELB Code Generator  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2015-2016 Free Software Foundation, Inc.
+;; Copyright (C) 2015-2018 Free Software Foundation, Inc.
 
 ;; Author: Chris Feng <chris.w.feng@gmail.com>
 
@@ -198,7 +198,7 @@ an `xelb-auto-padding' attribute."
       (princ (format "\
 ;;; xcb-%s.el --- X11 %s  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2015-2016 Free Software Foundation, Inc.
+;; Copyright (C) 2015-2018 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -263,17 +263,17 @@ an `xelb-auto-padding' attribute."
       (when xelb-error-alist
         (pp
          `(defconst ,(intern (concat xelb-prefix "error-number-class-alist"))
-            ',xelb-error-alist "(error-number . error-class) alist"))
+            ',xelb-error-alist "(error-number . error-class) alist."))
         (princ "\n"))
       (when xelb-event-alist
         (pp
          `(defconst ,(intern (concat xelb-prefix "event-number-class-alist"))
-            ',xelb-event-alist "(event-number . event-class) alist"))
+            ',xelb-event-alist "(event-number . event-class) alist."))
         (princ "\n"))
       (when xelb-xge-alist
         (pp
          `(defconst ,(intern (concat xelb-prefix "xge-number-class-alist"))
-            ',xelb-xge-alist "(xge-number . event-class) alist"))
+            ',xelb-xge-alist "(xge-number . event-class) alist."))
         (princ "\n"))
       ;; Print footer
       (princ (format "\
@@ -302,6 +302,7 @@ an `xelb-auto-padding' attribute."
     (`error (xelb-parse-error node))
     (`eventcopy (xelb-parse-eventcopy node))
     (`errorcopy (xelb-parse-errorcopy node))
+    (`eventstruct (xelb-parse-eventstruct node))
     ((or `comment `doc))                ;ignored
     (x (error "Unsupported top-level element: <%s>" x))))
 
@@ -494,6 +495,12 @@ The `combine-adjacent' attribute is simply ignored."
     `((defclass ,name (xcb:-error ,refname) ;Shadow the method of ref
         ((~code :initform ,error-number))))))
 
+(defun xelb-parse-eventstruct (node)
+  "Parse <eventstruct>."
+  (let ((name (intern (concat xelb-prefix (xelb-node-attr node 'name)))))
+    ;; Only conventional events are supported (and we don't check opcode).
+    `((defclass ,name (xcb:-event) nil))))
+
 ;;;; XCB: structure contents
 
 (defun xelb-parse-structure-content (node)
