@@ -1,6 +1,6 @@
 ;;; xcb.el --- X protocol Emacs Lisp Binding (XELB)  -*- lexical-binding: t -*-
 
-;; Copyright (C) 2015-2016 Free Software Foundation, Inc.
+;; Copyright (C) 2015-2018 Free Software Foundation, Inc.
 
 ;; Author: Chris Feng <chris.w.feng@gmail.com>
 
@@ -101,6 +101,17 @@
    (data :initarg :data :initform "" :type string))
   :documentation "X connection authentication info.")
 
+(cl-defmethod xcb:-get-extra-plist ((conn xcb:connection) module prop)
+  "Get the value of PROP from the extra plist for module MODULE."
+  (plist-get (plist-get (slot-value conn 'extra-plist) module) prop))
+
+(cl-defmethod xcb:-set-extra-plist ((conn xcb:connection) module prop val)
+  "Set the value of PROP in the extra plist for module MODULE to VAL."
+  (with-slots (extra-plist) conn
+    (setf extra-plist
+          (plist-put extra-plist module
+                     (plist-put (plist-get extra-plist module) prop val)))))
+
 (defun xcb:connect (&optional display _screen)
   "Connect to X server with display DISPLAY."
   (declare (advertised-calling-convention (&optional display) "25.1"))
@@ -692,7 +703,7 @@ The sequence number shall be returned by `xcb:+request-checked'."
   "Make a request and return its replies and errors.
 
 If MULTIPLE is nil, the return value is a list of which the car is the only
-reply and the cadr a list of errors. Otherwise, the car of the result is a
+reply and the cadr a list of errors.  Otherwise, the car of the result is a
 list of replies.
 
 WARNING: for requests that have multiple replies, you MUST supply a non-nil
